package de.dfki.km.academicresearch.eswc2016challenge.cameraready;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import org.jgrapht.graph.DirectedWeightedPseudograph;
import org.nustaq.serialization.FSTObjectInput;
import org.nustaq.serialization.FSTObjectOutput;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public abstract class JGraphTFinderBaseURI extends WeightedPathFinder {

    private static final Logger logger = LoggerFactory.getLogger(JGraphTFinderBaseURI.class);

    protected DirectedWeightedPseudograph<String, URIEdge> g;//String as Edge does not work because in  AbstractBaseGraph addEdge() checks containsEdge(e) in the whole graph

    public JGraphTFinderBaseURI() {
        g = new DirectedWeightedPseudograph<>(URIEdge.class);
    }

    @Override
    public void sendOnlyURI(String subject, String predicate, String object) {
        super.sendOnlyURI(subject, predicate, object);

        g.addVertex(subject);
        g.addVertex(object);

        g.addEdge(subject, object, new URIEdge(predicate, subject, object)); //since g  is a pseudograph then loop are okay
    }
    
    @Override
    public void sendEdgeWeight(String predicate, double weight) {
    	super.sendEdgeWeight(predicate, weight);    	
    	for(URIEdge e: g.edgeSet()){
    		if(e.getEdge().equals(predicate)){
    			g.setEdgeWeight(e, weight);
    		}
    	}
    }
    

    @Override
    public void startStreaming() throws Exception {
        //normal streaming needs 39.611ms
    }

    @Override
    public void endStreaming() throws Exception {
        super.endStreaming();
        File saveFile = getIndexFile();
        logger.debug("write to " + saveFile.getAbsolutePath());
        FSTObjectOutput out = null;
        try {
            out = new FSTObjectOutput(new FileOutputStream(saveFile));
            out.writeObject(g); //needs 11.578ms
        } finally {
            out.close();
        }
        logger.debug("done");
    }

    @Override
    public void loadIndex() throws Exception {
        super.loadIndex();

        File loadFile = getIndexFile();
        FSTObjectInput out = null;
        try {
            out = new FSTObjectInput(new FileInputStream(loadFile));
            logger.debug("start deserialize");
            g = (DirectedWeightedPseudograph<String, URIEdge>) out.readObject();   //needs 6.791ms -> GREAT!! much better than streaming 
            logger.debug("done");
        } finally {
            out.close();
        }
    }

}
