package de.dfki.km.academicresearch.eswc2016challenge.cameraready;

import java.awt.Component;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.Map;
import java.util.Queue;
import javax.swing.JTree;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.DefaultTreeModel;
import org.jgrapht.DirectedGraph;

/**
 *
 * @author Markus Schr&ouml;der
 */
public class JTreeFrame extends javax.swing.JFrame {

    private DataBasis data;
    
    public JTreeFrame(DataBasis data) {
        this.data = data;
        initComponents();
        
        jTree.setCellRenderer(new DefaultTreeCellRenderer() {

            @Override
            public Component getTreeCellRendererComponent(JTree tree, Object value, boolean selected, boolean expanded, boolean leaf, int row, boolean hasFocus) {
                DefaultMutableTreeNode mtn = (DefaultMutableTreeNode) value;
                DefaultTreeCellRenderer component = (DefaultTreeCellRenderer) super.getTreeCellRendererComponent(tree, value, selected, expanded, leaf, row, hasFocus);

                if(mtn.getUserObject() instanceof EdgeDelta) {
                    EdgeDelta ed = (EdgeDelta) mtn.getUserObject();
                    component.setText((ed.crossedgeToParent ? "-- " : "") + data.toString(ed.edge) + " (" +  ed.delta + ")");
                }
                
                return component;
            }
        
        });
    }

    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jScrollPane = new javax.swing.JScrollPane();
        jTree = new javax.swing.JTree();

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);

        javax.swing.tree.DefaultMutableTreeNode treeNode1 = new javax.swing.tree.DefaultMutableTreeNode("root");
        jTree.setModel(new javax.swing.tree.DefaultTreeModel(treeNode1));
        jScrollPane.setViewportView(jTree);

        getContentPane().add(jScrollPane, java.awt.BorderLayout.CENTER);

        pack();
    }// </editor-fold>//GEN-END:initComponents

    void showHeap(BinaryHeap<EdgeDelta> heap) {
        DefaultTreeModel model = (DefaultTreeModel) jTree.getModel();
        
        //parent queue
        //Queue<DefaultMutableTreeNode> qnode = new LinkedList<>();
        //qnode.add(null);
        
        Map<Integer, DefaultMutableTreeNode> id2node = new HashMap<>();
        //id2node.put(1, null);
        
        //queue of indices
        Queue<Integer> q = new LinkedList<>();
        q.add(1);//start
        
        while(!q.isEmpty()) {
            
            int index = q.poll();
            int parentIndex = heap.parentIndex(index);
            
            //not root
            if(parentIndex != 0) {
                
                DefaultMutableTreeNode child = new DefaultMutableTreeNode(heap.get(index));
                id2node.put(index, child);
                id2node.get(parentIndex).add(child);
                
            } else {
                model.setRoot(new DefaultMutableTreeNode(heap.get(index)));
                id2node.put(index, (DefaultMutableTreeNode) model.getRoot());
            }
            
            
            if(heap.hasLeftChild(index))
                q.add(heap.leftIndex(index));
            
            if(heap.hasRightChild(index))
                q.add(heap.rightIndex(index));
        }
        
        expandAllNodes();
    }
    
    void showGraph(DirectedGraph<EdgeDelta, Integer> PG) {
        DefaultTreeModel model = (DefaultTreeModel) jTree.getModel();
        
        Map<EdgeDelta, DefaultMutableTreeNode> ed2node = new HashMap<>();
        
        for(EdgeDelta v : PG.vertexSet()) {
            //root
            if(PG.incomingEdgesOf(v).isEmpty()) {
                model.setRoot(new DefaultMutableTreeNode(v));
                ed2node.put(v, (DefaultMutableTreeNode) model.getRoot());
            } else {
                DefaultMutableTreeNode node = new DefaultMutableTreeNode(v);
                ed2node.put(v, node);
            }
        }
        for(Integer edge : PG.edgeSet()) {
            ed2node.get(PG.getEdgeSource(edge)).add(ed2node.get(PG.getEdgeTarget(edge)));
        }
        
        expandAllNodes();
    }

    private void expandAllNodes(JTree tree, int startingIndex, int rowCount){
    for(int i=startingIndex;i<rowCount;++i){
        tree.expandRow(i);
    }

    if(tree.getRowCount()!=rowCount){
        expandAllNodes(tree, rowCount, tree.getRowCount());
    }
}

    private void expandAllNodes() {
        expandAllNodes(jTree, 0, jTree.getRowCount());
    }
    
    public static void show(BinaryHeap<EdgeDelta> heap, DataBasis data) {
        java.awt.EventQueue.invokeLater(() -> {
            JTreeFrame tf = new JTreeFrame(data);
            tf.showHeap(heap);
            tf.setVisible(true);
        });
    }
    
    public static void show(DirectedGraph<EdgeDelta, Integer> PG, DataBasis data) {
        java.awt.EventQueue.invokeLater(() -> {
            JTreeFrame tf = new JTreeFrame(data);
            tf.showGraph(PG);
            tf.setVisible(true);
        });
    }
    
    

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JScrollPane jScrollPane;
    private javax.swing.JTree jTree;
    // End of variables declaration//GEN-END:variables
}
