package de.dfki.km.academicresearch.eswc2016challenge.cameraready;

import com.google.common.collect.BiMap;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.PriorityQueue;
import java.util.Queue;
import java.util.Set;
import org.apache.jena.rdf.model.ResourceFactory;
import org.jgrapht.DirectedGraph;
import org.jgrapht.EdgeFactory;
import org.jgrapht.graph.DefaultDirectedGraph;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 *
 * @author Markus Schr&ouml;der
 */
public class ModifiedEppstein {
    
    private static final Logger logger = LoggerFactory.getLogger(ModifiedEppstein.class);
    
    //data basis
    private BiMap<String, Integer> uri2id;
    private int[] triplearray;
    public DataBasis data;
    
    private int targetId;
    
    private Set[] gIncomingEdgesOf;
    private DistAndEdge[] T;//position is uriId
    private Set<Integer> sidetracks;
    private Map<Integer, Byte> edge2delta;
    private Integer[] minSidetrackOf;
    private Set[] outgoingSidetracksOf;
    private BinaryHeap[] houtOf;
    private BinaryHeap[] htOf;
    private byte[] pgEdgeWeightOf;
    
    private boolean injection;
    private int injectSourceId;
    private Map<Integer, Integer> edge2injectEdge;
    
    public static final int INIT = -2;
    private int pgEdgeId = 0;
    
    private long beginInit;
    private long endInit;
    
    private long beginFind;
    private long endFind;
    
    public ModifiedEppstein(String target, DataBasis data) throws Exception {
        this.uri2id = data.getUri2id();
        this.triplearray = data.getTriplearray();
        this.data = data;
        
        //checkCorrectness();
        
        if(cleanupMemory) {
            logger.debug("init cleanup...");
            System.gc();
            PrintHeapUsage.printGB();
        }
        
        //ids
        targetId = uri2id.get(target);
        
        beginInit = System.currentTimeMillis();
        
        logger.debug("gIncomingEdgesOf...");
        gIncomingEdgesOf();
        PrintHeapUsage.printGB();
        
        logger.debug("T...");
        dijkstra();// v -> dist & shortest path edge
        PrintHeapUsage.printGB();
        
        //cleanup
        if(cleanupMemory) {
            gIncomingEdgesOf = null;
            cleanupMemory();
        }
        
        logger.debug("delta...");
        delta();
        PrintHeapUsage.printGB();
           
        logger.debug("hout...");
        hout();
        PrintHeapUsage.printGB();
        
        logger.debug("ht...");
        ht();
        PrintHeapUsage.printGB();
                
        //showHTOf("http://dbpedia.org/resource/Felipe_Massa");
        //minSidetrackOf("http://dbpedia.org/resource/Felipe_Massa");
        //T("http://dbpedia.org/resource/Felipe_Massa");
        
        if(cleanupMemory) {
            minSidetrackOf = null;
            cleanupMemory();
        
            outgoingSidetracksOf = null;
            cleanupMemory();
            
            
            PrintHeapUsage.printGB();
        }
        
        endInit = System.currentTimeMillis();
        
        logger.debug("init done in {} ms", endInit - beginInit);
    }
    
    public PathList find(String source, int k) throws Exception {
        logger.debug("{} {}", source, k);
        beginFind = System.currentTimeMillis();
        
        int sourceId = uri2id.get(source);
        List<Integer[]> paths = bfs(sourceId, k);
        
        endFind = System.currentTimeMillis();
        logger.debug("bfs done in {} ms", endFind - beginFind);
        
        return toPathList(paths);
    }
    
    public PathList find(String source, int k, int injectSourceId) throws Exception {
        logger.debug("{} {}", source, k);
        beginFind = System.currentTimeMillis();
        
        injection = true;
        this.injectSourceId = injectSourceId;
        edge2injectEdge = new HashMap<>();
        
        int sourceId = uri2id.get(source);
        List<Integer[]> paths = bfs(sourceId, k);
        
        logger.debug("bfs done");
        
        return toPathList(paths);
    }
    
    
    //==========================================================================
    
    private void cleanupMemory() {
        logger.debug("cleanup memory...");
        System.gc();
        PrintHeapUsage.printGB();
    }
    
    private void gIncomingEdgesOf() {
        gIncomingEdgesOf = new Set[uri2id.size()];
        for(int i = 0; i < gIncomingEdgesOf.length; i++) {
            gIncomingEdgesOf[i] = new HashSet<>();
        }
        for(int i = 0; i < triplearray.length; i += 3) {
            gIncomingEdgesOf[triplearray[i+2]].add(i);
        }
    }
    
    private void dijkstra() {
        //int maxDist = 0;
        
        Queue<Integer> q = new LinkedList<>();
        q.add(targetId);
        
        T = new DistAndEdge[uri2id.size()];
        T[targetId] = new DistAndEdge((byte)0, -1);
        
        //assume all are sidetracks
        sidetracks = new HashSet<>();
        for(int i = 0; i < triplearray.length; i += 3) {
            sidetracks.add(i);
        }
        
        while (!q.isEmpty()) {
            Integer cur = q.poll(); //we dont need to use a priority queue , because we have to visit all vertices
            
            for (Integer e : (Set<Integer>) gIncomingEdgesOf[cur]) {
                
                //neighbour -e-> cur
                
                //neighbour is the source
                int neighbour = triplearray[e+0];
                byte newDist = (byte)(T[cur].dist + (byte)1);
                
                //maxDist = Math.max(maxDist, newDist);
                
                //boolean itsNew = T[neighbour] == null;
                //boolean itsShorter = !itsNew && newDist < T[neighbour].dist;
                //tritt nie auf da distance immer 1
                
                if(T[neighbour] == null) {
                    T[neighbour] = new DistAndEdge(newDist, e);
                    
                    //because e is a shortest path edge
                    sidetracks.remove(e);
                    
                } else {
                    continue;//because of loop
                }
                
                q.add(neighbour);
            }
        }        
        //logger.debug("max distance is {}", maxDist); //max distance is 64
    }
    
    private void delta() {
        //int maxDelta = 0;
        
        minSidetrackOf = new Integer[uri2id.size()];//1551041 * 32 bits
        edge2delta = new HashMap<>();//int -> short
        outgoingSidetracksOf = new Set[uri2id.size()];
        for(int i = 0; i < outgoingSidetracksOf.length; i++) {
            outgoingSidetracksOf[i] = new HashSet<>();
        }
        
        for (Integer e : sidetracks) {

            Integer tail = triplearray[e+0];
            Integer head = triplearray[e+2];

            //unreachable: so infinity distance
            if(T[tail] == null || T[head] == null) {
                continue;
            }
            
            ////delta(e) = l(e) + d(head(e), t) - d(tail(e), t)
            byte delta_e = (byte)((byte)1 + (T[head].dist - T[tail].dist));

            //maxDelta = Math.max(maxDelta, delta_e);
            
            edge2delta.put(e, delta_e);
            
            if (minSidetrackOf[tail] == null || delta_e < edge2delta.get(minSidetrackOf[tail])) {
                minSidetrackOf[tail] = e;
            }
            
            outgoingSidetracksOf[tail].add(e);
        }
        //logger.debug("max delta is {}", maxDelta);//max delta is 49
    }
    
    private void hout() {
        
        houtOf = new BinaryHeap[uri2id.size()];//1551041
        
        for(int v = 0; v < outgoingSidetracksOf.length; v++) {
            
            //no sidetracks at all
            if(outgoingSidetracksOf[v] == null)
                continue;
            
            BinaryHeap<EdgeDelta> h = new BinaryHeap<>();
            
            for(Integer sidetrack : (Set<Integer>) outgoingSidetracksOf[v]) {
                
                //we know that it has one
                //if sidetrack is the minimal sidetrack: do not add
                if(Objects.equals(sidetrack, minSidetrackOf[v]))
                    continue;
                
                //if not containing than it has a infinite delta: never reachable
                if(edge2delta.containsKey(sidetrack)) {
                    h.add(new EdgeDelta(sidetrack, edge2delta.get(sidetrack)));
                }
            }
            
            if(!h.isEmpty())
                houtOf[v] = h;
        }
    }
    
    private void ht() {
        htOf = new BinaryHeap[uri2id.size()];//1551041
        
        for(int v = 0; v < uri2id.size(); v++) {
            //only those where we can reach the destination
            if(T[v] == null)
                continue;
            
            int cur = v;
            while (true) {
                Integer min = minSidetrackOf[cur];
                
                //if there is a minimal sidetrack
                if(min != null) {
                    
                    //lazy
                    if(htOf[v] == null)
                        htOf[v] = new BinaryHeap();
                    
                    htOf[v].add(new EdgeDelta(min, edge2delta.get(min)));
                }
                
                //cur must be in T that we can move forward
                if(T[cur] == null)
                    break;//here we can not go more
                
                int e = T[cur].edge;
                
                //target
                cur = triplearray[e+2];
            }
        }
    }
    
    private List<Integer[]> bfs(int sourceId, int k) {
        
        //data structures
        Queue<EdgeDelta> H = new PriorityQueue<>();
        DirectedGraph<EdgeDelta, Integer> PG = new DefaultDirectedGraph<>(Integer.class); //new MyGraph(); //TODO MyGraph
        pgEdgeWeightOf = new byte[Integer.MAX_VALUE / 8];//have to be the size of necessary edges for PG
        
        //init edge
        EdgeDelta init = new EdgeDelta(INIT, (byte)0);
        H.add(init);
        PG.addVertex(init);
        
        //the result
        List<Integer[]> paths = new ArrayList<>(k);
        
        int iteration = 0;
        int pruned = 0;
        
        //bfs while
        while (!H.isEmpty() && paths.size() < k) {
            EdgeDelta cur = H.remove();
            
            Result result = buildPath(sourceId, cur);
            
            //if path valid
            if(result.path != null) {
                //logger.debug(data.toString(result.path));
                paths.add(result.path);
            }
            
            //if we have to extend PG
            if(result.cont) {
                construct(cur, sourceId, PG);
            } else {
                pruned++;
            }
            
            //next iteration
            //TODO MyGraph
            for(Integer pgEdge : PG.outgoingEdgesOf(cur)) {
                EdgeDelta child = PG.getEdgeTarget(pgEdge);
            //for(EdgeDelta child : cur.children) {
                
                child.delta = (byte) (cur.delta + pgEdgeWeightOf[pgEdge]);//(byte) ((int) PG.getEdgeWeight(pgEdge)));
                //child.delta = (byte) (cur.delta + pgEdgeWeightOf[child.parentEdgeId]);
                
                //if(child.delta == Byte.MAX_VALUE) {
                //    logger.debug("delta is max");
                //}
                
                H.add(child);
            }
            
            PG.removeVertex(cur);
            
            iteration++;
            
            //memory cleanup
            if(logEvery && iteration % logEveryIteration == 0) {
                log(k, iteration, paths, pruned, H, PG);
            }
        }
        
        //JTreeFrame.show(PG, data);
        
        log(k, iteration, paths, pruned, H, PG);
        
        return paths;
    }
    
    private void construct(EdgeDelta cur, int sourceId, DirectedGraph<EdgeDelta, Integer> PG) {
        int v = (cur.edge == INIT) ? sourceId : triplearray[cur.edge+2];

        Set<EdgeDelta> newOnes = add(cur, htOf[v], PG, true);
            
        for(EdgeDelta newone : newOnes) {
            //hout of the tail of this edge
            add(newone, houtOf[triplearray[newone.edge]], PG, false);
        }
    }
    
    private Set<EdgeDelta> add(EdgeDelta root, BinaryHeap<EdgeDelta> heap, DirectedGraph<EdgeDelta, Integer> PG, boolean hT) {
        if(heap == null || heap.isEmpty())
            return new HashSet<>();
        
        Set<EdgeDelta> newOnes = new HashSet<>();
        
        //copy all
        EdgeDelta[] copy = new EdgeDelta[heap.length()];
        for(int i = 0; i < copy.length; i++) {
            if(heap.get(i) == null)
                continue;
            
            copy[i] = new EdgeDelta(heap.get(i));//this needs memory
            newOnes.add(copy[i]);
            
            //only the one in the heap
            PG.addVertex(copy[i]);
            
            if(newOnes.size() >= heap.size())
                break;
        }
        
        //queue of indices
        Queue<Integer> q = new LinkedList<>();
        q.add(1);//start
        
        while(!q.isEmpty()) {
            
            int index = q.poll();
            int parentIndex = heap.parentIndex(index);

            //decide the connection
            EdgeDelta a;
            EdgeDelta b = copy[index];
            //not root
            if(parentIndex != 0) {
                a = copy[parentIndex];
            } else {
                a = root;
                //if ht the root -> HT is a crossedge
                if(hT) {
                    b.crossedgeToParent = true;
                }
            }
            
            //make the connection
            PG.addEdge(a, b, pgEdgeId);
            //record incremental costs
            pgEdgeWeightOf[pgEdgeId++] = (hT && a == root) ? b.delta : (byte)(b.delta - a.delta);
            //parent
            if(a.edge != INIT)
                b.parent = a;
            
            if(heap.hasLeftChild(index))
                q.add(heap.leftIndex(index));
            
            if(heap.hasRightChild(index))
                q.add(heap.rightIndex(index));
        }
        
        return newOnes;
    }
    
    private Result buildPath(int sourceId, EdgeDelta origin) {
        
        Set<Integer> setForDuplicateCheck = new HashSet<>();
        LinkedList<EdgeDelta> sidetrackList = new LinkedList<>();
        
        //only check sidetracks for non-init edges
        if(origin.edge != INIT) {
            
            setForDuplicateCheck.add(origin.edge);
            sidetrackList.add(origin);

            //go parent, look for crossedge
            EdgeDelta parent;
            EdgeDelta cur = origin;
            while((parent = cur.parent) != null) {

                if(cur.crossedgeToParent)
                    if(!setForDuplicateCheck.add(parent.edge))
                        return new Result(false); //case: duplicate sidetrack
                    else
                        sidetrackList.addFirst(parent);

                cur = parent;
            }
        }
        
        //if we want to inject
        //and there are sidetracks
        //while the first sidetrack starts with sourceId (dummy)
        //this sidetrack will be used
        //thus we have to inject (injectSourceId, P, ...)
        boolean firstSidetrackStartsWithSource = !sidetrackList.isEmpty() && triplearray[sidetrackList.get(0).edge] == sourceId;
        if(injection && firstSidetrackStartsWithSource) {
            
            //find the edge id
            int edgeId;
            
            if(edge2injectEdge.containsKey(sidetrackList.get(0).edge)) {
                edgeId = edge2injectEdge.get(sidetrackList.get(0).edge);
            } else {
                //find
                for(edgeId = 0; edgeId < triplearray.length; edgeId += 3) {
                    if(triplearray[edgeId+0] == injectSourceId && 
                       triplearray[edgeId+1] == triplearray[sidetrackList.get(0).edge+1] && 
                       triplearray[edgeId+2] == triplearray[sidetrackList.get(0).edge+2]) {
                        break;
                    }
                }
                
                if(edgeId >= triplearray.length)
                    throw new RuntimeException("edge not found");
                
                //store for next time
                edge2injectEdge.put(sidetrackList.get(0).edge, edgeId);
            }
            
            if(!setForDuplicateCheck.add(edgeId))
                return new Result(false); //case: duplicate sidetrack
            
        }
        
        //logger.debug("sidetracks: {}", data.toString(sidetrackList));
        
        //array for faster access
        EdgeDelta[] sidetrackArray = sidetrackList.toArray(new EdgeDelta[0]);
        int sidetrackIndex = 0;
        
        setForDuplicateCheck.clear();
        
        //if we want to inject
        //and the first sidetrack starts NOT with source
        //we must use a shortest path edge
        //thus, we inject the correct shortest path edge
        if(injection && !firstSidetrackStartsWithSource) {
            int spt = T[sourceId].edge;
            
            //find the edge id
            int edgeId;
            
            if(edge2injectEdge.containsKey(spt)) {
                edgeId = edge2injectEdge.get(spt);
            } else {
                //find
                for(edgeId = 0; edgeId < triplearray.length; edgeId += 3) {
                    if(triplearray[edgeId+0] == injectSourceId && 
                       triplearray[edgeId+1] == triplearray[spt+1] && 
                       triplearray[edgeId+2] == triplearray[spt+2]) {
                        break;
                    }
                }
                
                if(edgeId >= triplearray.length)
                    throw new RuntimeException("edge not found");
            
                edge2injectEdge.put(spt, edgeId);
            }
            
            setForDuplicateCheck.add(edgeId);
        }
        
        List<Integer> path = new ArrayList<>(50);
        
        int cur = sourceId;
        while(!(sidetrackIndex >= sidetrackArray.length && cur == targetId)) {
            int track;
            
            if(sidetrackIndex < sidetrackArray.length && triplearray[sidetrackArray[sidetrackIndex].edge+0] == cur) { //nextSidetracks.get(nextSidetrackIndex).getSource().equals(cur)
                track = sidetrackArray[sidetrackIndex++].edge;
            } else {
                track = T[cur].edge;
            }
            
            if(!setForDuplicateCheck.add(track))
                return new Result(true); //case: duplicate shortest path edge
            
            path.add(track);
            
            cur = triplearray[track+2];
        }
        
        //case: path found and continue
        return new Result(path.toArray(new Integer[0]), true);
    }
    
    private PathList toPathList(List<Integer[]> paths) throws Exception {
        PathList pl = new PathList();
        
        BiMap<Integer, String> id2uri = uri2id.inverse();
        
        for(Integer[] path : paths) {
            Path p = new Path();
            
            //p.add(ResourceFactory.createResource(id2uri.get(triplearray[path[0]])));
            
            for(int i = 0; i < path.length; i++) {
                
                Integer[] uriIds = new Integer[] { triplearray[path[i]+0], triplearray[path[i]+1] };
                    
                p.add(ResourceFactory.createResource(id2uri.get(uriIds[0])));
                p.add(ResourceFactory.createProperty(id2uri.get(uriIds[1])));
                
                //last
                if(i == path.length-1) {
                    p.add(ResourceFactory.createResource(id2uri.get(triplearray[path[i]+2])));
                }
            }
            
            pl.addWithOrderCheck(p);
        }
        
        return pl;
    } 
    
    //==========================================================================
    //debug kram
    
    @Deprecated
    private void checkCorrectness() {
        
        Integer[] a = new Integer[] { triplearray[83970528+0], triplearray[83970528+1], triplearray[83970528+2] };
        Integer[] b = new Integer[] { triplearray[54795864+0], triplearray[54795864+1], triplearray[54795864+2] };
        
        logger.debug("{} {}", Arrays.toString(a), a.hashCode());
        logger.debug("{} {}", Arrays.toString(b), b.hashCode());
        //[45351, 1547502, 149921] 1709537756
        //[45351, 1547502, 149921] 2101842856
        //same array content but different hashCodes?!
        
        logger.debug("{} {}", Arrays.toString(a), Arrays.hashCode(a));
        logger.debug("{} {}", Arrays.toString(b), Arrays.hashCode(b));
        
        
        
        Set<Integer[]> setForDuplicateCheck = new HashSet<>();
        for(int i = 0; i < triplearray.length; i += 3) {
            if(!setForDuplicateCheck.add(new Integer[] { triplearray[i+0], triplearray[i+1], triplearray[i+2] })) {
                logger.debug("duplicate edge detected: {}", data.toString(i));
                return;
            }
        }
    }
    
    //minSidetrackOf(http://dbpedia.org/resource/Felipe_Massa) = r/Felipe_Massa p/after r/Robert_Kubica [83970528]
    //T(http://dbpedia.org/resource/Felipe_Massa) = r/Felipe_Massa p/after r/Robert_Kubica [54795864] (3)
    
    private void showHTOf(String uri) {
        JTreeFrame.show(htOf[uri2id.get(uri)], data);
    }
    
    private void minSidetrackOf(String uri) {
        int e = minSidetrackOf[uri2id.get(uri)];
        logger.debug("minSidetrackOf({}) = {}[{}]", uri, data.toString(e), e);
        logger.debug("minSidetrackOf({}) = {} {} {}", uri2id.get(uri), triplearray[e+0], triplearray[e+1],triplearray[e+2]);
    }
    
    private void T(String uri) {
        DistAndEdge de = T[uri2id.get(uri)];
        logger.debug("T({}) = {}[{}] ({})", uri, data.toString(de.edge), de.edge, de.dist);
        logger.debug("T({}) = {} {} {}", uri2id.get(uri), triplearray[de.edge+0], triplearray[de.edge+1],triplearray[de.edge+2]);
    }
    
    private final int logEveryIteration = 1000;
    private final boolean logEvery = false;
    private final boolean cleanupMemory = false;
    
    private void log(int k, int iteration, List<Integer[]> paths, int pruned, Queue<EdgeDelta> H, DirectedGraph<EdgeDelta, Integer> PG) {
        
        endFind = System.currentTimeMillis();
        
        float overhead = (iteration - k) / (float) k;
        logger.debug("k={}, iterations={}, overhead={}, found={}, time={} ms, init+find time={} ms, pruned={}, PG(V)={}, PG(E)={}, H={}, {}", 
                k, 
                iteration, 
                overhead, 
                paths.size(), 
                (endFind - beginFind),
                (endInit - beginInit) + (endFind - beginFind),
                pruned, 
                PG.vertexSet().size(), 
                PG.edgeSet().size(), 
                H.size(), 
                PrintHeapUsage.getInfoGB()
        );
    }
    
    //==========================================================================
    
    class DistAndEdge {
        byte dist;
        int edge;

        public DistAndEdge(byte dist, int edge) {
            this.dist = dist;
            this.edge = edge;
        }
        
    }
    
    class Result {
        Integer[] path;
        boolean cont; //true if p is valid or p contains multiple shortest path edges

        public Result(boolean cont) {
            this.cont = cont;
        }
        
        public Result(Integer[] path, boolean cont) {
            this.path = path;
            this.cont = cont;
        }
    }
    
    class MyGraph implements DirectedGraph<EdgeDelta, Integer> {

        private Set<EdgeDelta> V;
        private Set<Integer> E;
        
        private Map<Integer, EdgeDelta> edgeId2target;
        private Map<EdgeDelta, Set<Integer>> source2edges;
        
        public MyGraph() {
            V = new HashSet<>();
            E = new HashSet<>();
            edgeId2target = new HashMap<>();
            source2edges = new HashMap<>();
        }
        
        @Override
        public boolean addVertex(EdgeDelta v) {
            return V.add(v);
        }
        
        @Override
        public boolean removeVertex(EdgeDelta v) {
            return V.remove(v);
        }
        
        @Override
        public Set<EdgeDelta> vertexSet() {
            return V;
        }
        
        @Override
        public Set<Integer> edgeSet() {
            return E;
        }

        
        @Override
        public boolean addEdge(EdgeDelta sourceVertex, EdgeDelta targetVertex, Integer e) {
            //edgeId2target.put(e, targetVertex);
            sourceVertex.children.add(targetVertex);
            
            if(sourceVertex.edge != INIT) {
                targetVertex.parent = sourceVertex;
                targetVertex.parentEdgeId = e;
            }
            
            //if(!source2edges.containsKey(sourceVertex)) {
            //    source2edges.put(sourceVertex, new HashSet<>(Arrays.asList(e)));
            //} else {
            //    source2edges.get(sourceVertex).add(e);
            //}
            
            //E.add(e);
            
            return true;
        }
        
        @Override
        public Set<Integer> outgoingEdgesOf(EdgeDelta vertex) {
            throw new RuntimeException("not implemented yet");
            
            //Set<Integer> edges = source2edges.get(vertex);
            //if(edges == null)
            //    return new HashSet<>();
            //return edges;
        }
        
        @Override
        public EdgeDelta getEdgeTarget(Integer e) {
            throw new RuntimeException("not implemented yet");
            //return edgeId2target.get(e);
        }


        
        //======================================================================

        @Override
        public int inDegreeOf(EdgeDelta vertex) {
            throw new RuntimeException("not implemented yet");
        }

        @Override
        public Set<Integer> incomingEdgesOf(EdgeDelta vertex) {
            throw new RuntimeException("not implemented yet");
        }

        @Override
        public int outDegreeOf(EdgeDelta vertex) {
            throw new RuntimeException("not implemented yet");
        }

        @Override
        public Set<Integer> getAllEdges(EdgeDelta sourceVertex, EdgeDelta targetVertex) {
            throw new RuntimeException("not implemented yet");
        }

        @Override
        public Integer getEdge(EdgeDelta sourceVertex, EdgeDelta targetVertex) {
            throw new RuntimeException("not implemented yet");
        }

        @Override
        public EdgeFactory<EdgeDelta, Integer> getEdgeFactory() {
            throw new RuntimeException("not implemented yet");
        }

        @Override
        public Integer addEdge(EdgeDelta sourceVertex, EdgeDelta targetVertex) {
            throw new RuntimeException("not implemented yet");
        }

        @Override
        public boolean containsEdge(EdgeDelta sourceVertex, EdgeDelta targetVertex) {
            throw new RuntimeException("not implemented yet");
        }

        @Override
        public boolean containsEdge(Integer e) {
            throw new RuntimeException("not implemented yet");
        }

        @Override
        public boolean containsVertex(EdgeDelta v) {
            throw new RuntimeException("not implemented yet");
        }
        
        @Override
        public Set<Integer> edgesOf(EdgeDelta vertex) {
            throw new RuntimeException("not implemented yet");
        }

        @Override
        public boolean removeAllEdges(Collection<? extends Integer> edges) {
            throw new RuntimeException("not implemented yet");
        }

        @Override
        public Set<Integer> removeAllEdges(EdgeDelta sourceVertex, EdgeDelta targetVertex) {
            throw new RuntimeException("not implemented yet");
        }

        @Override
        public boolean removeAllVertices(Collection<? extends EdgeDelta> vertices) {
            throw new RuntimeException("not implemented yet");
        }

        @Override
        public Integer removeEdge(EdgeDelta sourceVertex, EdgeDelta targetVertex) {
            throw new RuntimeException("not implemented yet");
        }

        @Override
        public boolean removeEdge(Integer e) {
            throw new RuntimeException("not implemented yet");
        }

        
        
        @Override
        public EdgeDelta getEdgeSource(Integer e) {
            throw new RuntimeException("not implemented yet");
        }

        
        @Override
        public double getEdgeWeight(Integer e) {
            throw new RuntimeException("not implemented yet");
        }
        
    }
}
