package de.dfki.km.academicresearch.eswc2016challenge.cameraready;

import java.util.LinkedList;
import java.util.List;
import static java.util.stream.Collectors.toList;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 *
 * @author Markus Schr&ouml;der
 */
public class ModifiedEppsteinBidirectional {
    
    private static final Logger logger = LoggerFactory.getLogger(ModifiedEppsteinBidirectional.class);
    
    private String source;
    private String target;
    private String P;
    private int k;
    private DataBasis data;
    
    public ModifiedEppsteinBidirectional(String source, String target, String P, int k, DataBasis data) throws Exception {
        this.source = source;
        this.target = target;
        this.P = P;
        this.k = k;
        this.data = data;
    }
    
    public PathList find() throws Exception {
        
        //t -> s
        logger.debug("t -> s");
        ModifiedEppsteinP modeppTS = new ModifiedEppsteinP(target, source, P, k, data.extend(true, new int[0]));
        PathList ts = modeppTS.find().reverseAll();
        //clean
        modeppTS = null;
        System.gc();
        PrintHeapUsage.printGB();
        
        //s -> t
        logger.debug("s -> t");
        ModifiedEppsteinP modeppST = new ModifiedEppsteinP(source, target, P, k, data);
        PathList st = modeppST.find();
        //clean
        modeppST = null;
        System.gc();
        PrintHeapUsage.printGB();
        
        return zip(st, ts);
    }
    
    private PathList zip(PathList a, PathList b) throws Exception {
        
        List<Path> al = a.getMap().values().stream().flatMap(s -> s.stream()).collect(toList());
        List<Path> bl = b.getMap().values().stream().flatMap(s -> s.stream()).collect(toList());
        
        List<Path> zip = zip(al, bl, k);
        
        return new PathList(zip);
    }
    
    private List<Path> zip(List<Path> p1, List<Path> p2, int k) {
        List<Path> result = new LinkedList<>();
            
        int i = 0;
        int j = 0;
        while(result.size() < k && (i < p1.size() || j < p2.size())) {

            int comp;
            Path left = null;
            Path right = null;

            //left is empty but right is full
            if(i >= p1.size() && j < p2.size()) {

                comp = +1;
                right = p2.get(j);
            }
            //left is full but right is empty
            else if(i < p1.size() && j >= p2.size()) {

                comp = -1;
                left = p1.get(i);

            } else {
                //both are full
                left  = p1.get(i);
                right = p2.get(j);
                comp = left.compareTo(right);

                if(left.equals(right)) {
                    comp = -1;
                    j++;
                } 
            }

            if(comp < 0) {
                if(!result.contains(left)) {
                    result.add(left);
                }
                i++;
            } else if(comp > 0) {
                if(!result.contains(right)) {
                    result.add(right);
                }
                j++;
            } else {

                if(!result.contains(left)) {
                    result.add(left);
                }
                i++;

                if(!result.contains(right)) {
                    result.add(right);
                }
                j++;
            }
        }
        return result;
    }
    
}
