package de.dfki.km.academicresearch.eswc2016challenge.cameraready;

import java.io.File;
import java.util.Iterator;
import java.util.List;
import org.apache.jena.atlas.lib.Sink;
import org.apache.jena.graph.Triple;

/**
 *
 * IF the sink closes e.g. flush then write you index to a specific file
 * Otherwise the loadIndex() method is called and then the Path finder should load the saved index
 * @author Markus Schr&ouml;der
 */
public abstract class PathFinder implements Sink<Triple>{
    
	
	protected String indexDir;
	protected String indexDataSetId;
	protected String finderName = this.getClass().getSimpleName();
	
	protected long vertexCount = 1;
	protected long edgeCount = 1;
	
	
	public void setVertexCount(long count){
		this.vertexCount = count;
	}
	
	public void setEdgeCount(long count){
		this.edgeCount = count;
	}
	
	public void setIndex(String dir, String dataSetId){
		this.indexDir = dir;
		this.indexDataSetId = dataSetId;
	}
	
	protected String getIndexExtension(){
		return ".index";
	}
	
	/***
	 * Gets the file where the pathfinder can store indices. The fileId should start at 0.
	 * @param fileId should start at 0
	 * @return
	 */
	protected File getIndexFile(int fileId){
		File f = new File(indexDir, finderName + "_" + indexDataSetId + "_"+fileId + getIndexExtension());
		return f;
	}
	
	/**
	 * For all PathFinders who only need one index File
	 * @return
	 */
	protected File getIndexFile(){
		return getIndexFile(0);
	}
		
	public boolean hasIndex(){
		return getIndexFile().exists();
	}
		
	    
    
	@Override
	public void flush() {	
		try {
			endStreaming();
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	@Override
	public void close() { 
		
	}	
	
	
	//You should override the following methods
	
	public abstract List<Path> find(String srcURI, String dstURI, int k, String startOrEndProp) throws Exception;
	
	
	public Iterator<? extends Path> find(String srcURI, String dstURI, String startOrEndProp) throws Exception{
		
		return find(srcURI, dstURI, 20, startOrEndProp).iterator();
	}
	
	
	protected long tripleCount = 0;
	@Override
	public void send(Triple item) {	
		tripleCount++;
		//TODO: maybe check the first triple and call startStreaming -> currently done in tests
    }
	
	public void loadIndex()throws Exception{
		
	}
	
	public void endStreaming()throws Exception{
		
	}
	
	public void startStreaming()throws Exception{
		
	}
    
}
