package de.dfki.km.academicresearch.eswc2016challenge.cameraready;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;
import java.util.TreeMap;
import org.apache.commons.io.IOUtils;
import org.apache.jena.rdf.model.Resource;

public class PathList {

    //private List<Set<Path>> pathes;
    private TreeMap<Integer, Set<Path>> map;

   
    
    
    public static List<Path> toPathList(File f) throws Exception{
    	List<Path> list = new LinkedList<>();
        try (BufferedReader br = new BufferedReader(new FileReader(f))) {
            String line;
            while ((line = br.readLine()) != null) {
                list.add(new Path(line));
            }
        }
        return list;
    }

    public PathList(File f) throws Exception {
    	this(toPathList(f));
    }
    
    public PathList(List<Path> pathes) throws Exception {

        map = new TreeMap<>();

        for (Path p : pathes) {
            addWithOrderCheck(p);
        }
    }

    public PathList() {
        map = new TreeMap<>();
    }

    public PathList replaceEveryFirstResourceWith(Resource res) {
        for(Set<Path> pathSet : map.values()) {
            for(Path path : pathSet) {
                if(!path.isEmpty())
                    path.set(0, res);
            }
        }
        return this;
    }
    
    public PathList reverseAll() {
        for(Set<Path> pathSet : map.values()) {
            for(Path path : pathSet) {
                path.reverse();
            }
        }
        return this;
    }
    
    public void addWithOrderCheck(Path p) throws Exception {

        int length = p.length();
        int lastkey = Integer.MIN_VALUE;
        if (map.isEmpty() == false) {
            lastkey = map.lastKey();
        }

        if (lastkey > length) {
            throw new Exception("Order is not correct");
        }

        Set<Path> pathes = map.getOrDefault(length, new HashSet<>());
        pathes.add(p);
        map.put(length, pathes);
    }

    @Override
    public int hashCode() {
        final int prime = 31;
        int result = 1;
        result = prime * result + ((map == null) ? 0 : map.hashCode());
        return result;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        PathList other = (PathList) obj;
        if (map == null) {
            if (other.map != null) {
                return false;
            }
        } else if (!map.equals(other.map)) {
            return false;
        }
        return true;
    }

    public TreeMap<Integer, Set<Path>> getMap() {
        return map;
    }
    
    private static final String newline = System.getProperty("line.separator"); 
    @Override
    public String toString() {
    	StringBuilder sb = new StringBuilder();
    	for(Set<Path> pathes : map.values()){
    		for(Path p : pathes){
    			sb.append(p.toString());
    			sb.append(newline);
    		}
    		sb.append("--");
    		sb.append(newline);
    	}
    	return sb.toString();
    }
    
    
    public String toSourceCode(String dataSet){
    	
    	StringBuilder sb = new StringBuilder();
    	for(Set<Path> pathes : map.values()){    		
    		for(Path p : sortPath(pathes)){
    			sb.append(p.toSourceCode(dataSet));
    			sb.append(newline);
    		}
    		sb.append(newline);
    	}
    	return sb.toString();
    	
    }
    
    
    public String toComparableString() {
    	StringBuilder sb = new StringBuilder();
    	for(Set<Path> pathes : map.values()){    		
    		//sort pathes for comparable file
    		for(Path p : sortPath(pathes)){
    			sb.append(p.toString());
    			sb.append(newline);
    		}
    		sb.append("--");
    		sb.append(newline);
    	}
    	return sb.toString();
    }
    
    private List<Path> sortPath(Set<Path> set){
    	
    	List<Path> list = new ArrayList<>(set);
    	Collections.sort(list, new PathComparator());
    	return list;
    }
    
    class PathComparator implements Comparator<Path> {
        @Override
        public int compare(Path a, Path b) {
        	
            //FIRST: compare length
        	
        	int k = Integer.compare(a.length(), b.length());        	
        	if (k != 0) return k;
        	
        	//same length
        	//SECOND: compare each of the element
        	
        	List<Resource> alist = a.getPath();
        	List<Resource> blist = b.getPath();
        	
        	for(int i=0; i < alist.size(); i++){
        		
        		Resource ra = alist.get(i);
        		Resource rb = blist.get(i);
        		
        		if(ra.isURIResource() == false || rb.isURIResource() == false)
        			continue;
        		
        		
        		int j = ra.getURI().compareTo(rb.getURI());
        		if(j != 0) return j; 
        	}
        	return 0;
        }
    }
    
    public void save(String filename) throws IOException {
        FileWriter fw = new FileWriter(new File(filename));
        IOUtils.write(toString(), fw);
        fw.close();
    }
}
