package de.dfki.km.academicresearch.eswc2016challenge.unittests.exampleTestTaskOne;

import de.dfki.km.academicresearch.eswc2016challenge.cameraready.Path;
import de.dfki.km.academicresearch.eswc2016challenge.cameraready.PathFinder;
import de.dfki.km.academicresearch.eswc2016challenge.cameraready.PathList;
import de.dfki.km.academicresearch.eswc2016challenge.unittests.ExampleDataset;
import de.dfki.km.academicresearch.eswc2016challenge.unittests.PathChooser;
import de.dfki.km.academicresearch.eswc2016challenge.unittests.Util;
import java.util.LinkedList;
import java.util.List;
import org.apache.jena.rdf.model.Model;
import org.apache.jena.riot.system.StreamOps;
import org.apache.jena.riot.system.StreamRDF;
import org.apache.jena.riot.system.StreamRDFLib;
import org.junit.BeforeClass;
import org.junit.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * 
 * This is a multigraph example
 * @author Sven
 *
 */
public class ExampleTests6 {
	
	private static final Logger logger = LoggerFactory.getLogger(ExampleTests6.class);

    private static Model datasetD6 = ExampleDataset.getDataSetD6();
    
    
  //=Choose Implementation=========================================================================
    private static PathFinder pathFinder;
    
    @BeforeClass
    public static void onceExecutedBeforeAll() throws Exception {
        logger.debug("@BeforeClass: onceExecutedBeforeAll");
        
        pathFinder = PathChooser.getCurrentPathFinder();
        
        pathFinder.setIndex("challenge_training_dataset", "datasetD6");
        if(pathFinder.hasIndex()){        	
        	pathFinder.loadIndex(); 
        }
        else{
            StreamRDF streamRDF = StreamRDFLib.sinkTriples(pathFinder); 
            pathFinder.startStreaming();
            StreamOps.graphToStream(datasetD6.getGraph(), streamRDF);   
        }
    }
    //========================================================================== 

    //@Test
    public void TaskOneTestOne() throws Exception {
    	
    	//           u2
    	//         | ^^
    	//      p3 | || p4,5
    	//         v ||
    	// A -p1->  u1  -p2-> B
    		
        List<Path> expected = new LinkedList<>();
        expected.add(new Path(ExampleDataset.A, ExampleDataset.p1, ExampleDataset.u1, ExampleDataset.p2, ExampleDataset.B));
        expected.add(new Path(ExampleDataset.A, ExampleDataset.p1, ExampleDataset.u1, ExampleDataset.p5, ExampleDataset.u2, ExampleDataset.p3, ExampleDataset.u1, ExampleDataset.p2, ExampleDataset.B));
        expected.add(new Path(ExampleDataset.A, ExampleDataset.p1, ExampleDataset.u1, ExampleDataset.p4, ExampleDataset.u2, ExampleDataset.p3, ExampleDataset.u1, ExampleDataset.p2, ExampleDataset.B));
        
        List<Path> actual = pathFinder.find(ExampleDataset.A.getURI(), ExampleDataset.B.getURI(), 3, null);
        
        Util.equalsSlow(new PathList(expected), new PathList(actual));
    }
    
    @Test
    public void TaskOneTestTwo() throws Exception {
    		
        List<Path> expected = new LinkedList<>();
        expected.add(new Path(ExampleDataset.A, ExampleDataset.p1, ExampleDataset.u1, ExampleDataset.p2, ExampleDataset.B));
        expected.add(new Path(ExampleDataset.A, ExampleDataset.p1, ExampleDataset.u1, ExampleDataset.p5, ExampleDataset.u2, ExampleDataset.p3, ExampleDataset.u1, ExampleDataset.p2, ExampleDataset.B));
        expected.add(new Path(ExampleDataset.A, ExampleDataset.p1, ExampleDataset.u1, ExampleDataset.p4, ExampleDataset.u2, ExampleDataset.p3, ExampleDataset.u1, ExampleDataset.p2, ExampleDataset.B));
        
        List<Path> actual = pathFinder.find(ExampleDataset.A.getURI(), ExampleDataset.B.getURI(), 4, null); //one more which is not possible
        
        Util.equalsSlow(new PathList(expected), new PathList(actual));
    }

}
