package de.dfki.km.academicresearch.eswc2016challenge.unittests.exampleTestTaskOne;

import de.dfki.km.academicresearch.eswc2016challenge.cameraready.Path;
import de.dfki.km.academicresearch.eswc2016challenge.cameraready.PathFinder;
import de.dfki.km.academicresearch.eswc2016challenge.cameraready.PathList;
import de.dfki.km.academicresearch.eswc2016challenge.cameraready.PrintHeapUsage;
import de.dfki.km.academicresearch.eswc2016challenge.unittests.ConfusionMatrix;
import de.dfki.km.academicresearch.eswc2016challenge.unittests.InputParam;
import de.dfki.km.academicresearch.eswc2016challenge.unittests.PathChooser;
import de.dfki.km.academicresearch.eswc2016challenge.unittests.TaskTest.TaskOne;
import de.dfki.km.academicresearch.eswc2016challenge.unittests.WriteComparableFile;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.Arrays;
import java.util.Collection;
import java.util.List;
import org.apache.jena.riot.RDFDataMgr;
import org.apache.jena.riot.system.StreamRDF;
import org.apache.jena.riot.system.StreamRDFLib;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.junit.runners.Parameterized;
import org.junit.runners.Parameterized.Parameters;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

@RunWith(Parameterized.class)
public class SubsetTask {

    private static final Logger logger = LoggerFactory.getLogger(SubsetTask.class);
    
    //=Choose Implementation=========================================================================
    private static PathFinder pathFinder;
    
    @Before
    public void init() throws Exception {
    	String task = String.format("task1_q%s_%s", query, k);
        logger.debug("@Before test {}.txt", task);
        
        PrintHeapUsage.gcAndprintGB();
        pathFinder = PathChooser.getCurrentPathFinder();
        
        pathFinder.setIndex("challenge_training_dataset", task);
        if(pathFinder.hasIndex()){        	
        	pathFinder.loadIndex(); 
        }
        else{
            StreamRDF streamRDF = StreamRDFLib.sinkTriples(pathFinder);   
            pathFinder.startStreaming();
            RDFDataMgr.parse(streamRDF, String.format("./subset/%s.nt",task)); 
        }
        PrintHeapUsage.gcAndprintGB();
        logger.debug("Finished Importing test {}.txt", task);
    }
    //========================================================================== 
    

    @Parameters(name = "Test {index}:  task1_q{0}_{1}.txt")
    public static Collection<Object[]> data() {
        return Arrays.asList(new Object[][]{
            
            {1, 8}, //OK 2016-02-28
            //{1, 344}, //OK 2016-02-28           
            //{1, 1068}, //OK 2016-02-28 (duplicate with bitmap modulo should be 1.000.000)
            //{1, 20152}, //OK 2016-03-01

            //{2, 3},//OK 2016-02-28
            //{2, 4},//OK 2016-02-28
            //{2, 79},//OK 2016-02-28
            //{2, 154},//OK 2016-02-28
            
            //{3, 36},//OK 2016-02-28
            //{3, 336},//OK 2016-02-28
            //{3, 4866},//OK 2016-02-28
            
            //{4, 2},//OK 2016-02-28
            //{4, 16},//OK 2016-02-28
            //{4, 250},//OK 2016-02-28
            //{4, 1906},//OK 2016-02-28
            //{4, 20224},//OK 2016-02-28
            //{4, 175560} //OK 2016-03-01 (81s), iteration: 192000 / found pathes: 175228 / extra: 16772 / k: 175560 / nonConstructed: 7573 / used MB: 2830 / P(G).vertices 5529063 / H 372211 / invalid type statistic: {Unkown=0, ShortestPath=9199, NeverReachDst=0, None=175228, PropertyPathBcOfSidetrack=0, Sidetrack=7573, PropertyPathBcOfShortestPath=0}
            
        });
    }    
    
    

    private int query;
    private int k;    
    
    public SubsetTask(int query, int k) {
        this.query = query;
        this.k = k;
    }

    @Test
    public void test() throws FileNotFoundException, IOException, Exception {

    	String task = "task1_q" + query + "_" + k;
    	
        //load file
        File f = new File("./challenge_training_result_sets/" + task + ".txt");
        Assert.assertTrue("File with expected results should be existing", f.exists());

        //parameters
        InputParam params = TaskOne.queryToParam.get(query);
        String start = params.getStart();
        String end = params.getEnd();

        //actual and expected
        List<Path> actual = pathFinder.find(start, end, k, null);

        PathList listExpected = new PathList(f);
        PathList listActual = new PathList(actual);
        
        WriteComparableFile.writeSingleFile(new File("./challenge_training_dataset/"), task, listExpected, listActual);
        WriteComparableFile.writeFileToAll(new File("./challenge_training_dataset/"), task, listExpected, listActual);
        
        //Util.equalsSlow(listExpected, listActual);
        
        ConfusionMatrix m = new ConfusionMatrix();
        m.compare(listExpected, listActual);
        m.writeSingleFile(new File("./challenge_training_dataset/"), task);
        m.printConsole(task);
        m.assertIt();
    }
    
}
