package de.dfki.km.academicresearch.eswc2016challenge.unittests.exampleTestTaskTwo;

import de.dfki.km.academicresearch.eswc2016challenge.cameraready.Path;
import de.dfki.km.academicresearch.eswc2016challenge.cameraready.PathFinder;
import de.dfki.km.academicresearch.eswc2016challenge.cameraready.PathList;
import de.dfki.km.academicresearch.eswc2016challenge.unittests.ExampleDataset;
import de.dfki.km.academicresearch.eswc2016challenge.unittests.PathChooser;
import de.dfki.km.academicresearch.eswc2016challenge.unittests.Util;
import java.util.LinkedList;
import java.util.List;
import org.apache.jena.rdf.model.Model;
import org.apache.jena.riot.system.StreamOps;
import org.apache.jena.riot.system.StreamRDF;
import org.apache.jena.riot.system.StreamRDFLib;
import org.junit.BeforeClass;
import org.junit.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class ExampleTests1 {

    private static final Logger logger = LoggerFactory.getLogger(ExampleTests1.class);

    private static Model datasetD1 = ExampleDataset.getDataSetD1();
    
    
  //=Choose Implementation=========================================================================
    private static PathFinder pathFinder;
    
    @BeforeClass
    public static void onceExecutedBeforeAll() throws Exception {
        logger.debug("@BeforeClass: onceExecutedBeforeAll");
        
        //pathFinder = new GraphML();
        pathFinder = PathChooser.getCurrentPathFinder();
        
        pathFinder.setIndex("challenge_training_dataset", "datasetD1");
        if(pathFinder.hasIndex()){        	
        	pathFinder.loadIndex(); 
        }
        else{
            StreamRDF streamRDF = StreamRDFLib.sinkTriples(pathFinder);  
            pathFinder.startStreaming();
            StreamOps.graphToStream(datasetD1.getGraph(), streamRDF);   
        }
    }
   

    //============= task two ===================================================
    
    @Test
    public void TaskTwoTestOne() throws Exception {

    	//k = 2
        //Start or End with P
        //A nach B    	
        List<Path> expected = new LinkedList<>();
        expected.add(new Path(ExampleDataset.A, ExampleDataset.P, ExampleDataset.u3, ExampleDataset.p7, ExampleDataset.B));
        expected.add(new Path(ExampleDataset.A, ExampleDataset.p3, ExampleDataset.u6, ExampleDataset.P, ExampleDataset.B));
    
        List<Path> actual = pathFinder.find(ExampleDataset.A.getURI(), ExampleDataset.B.getURI(), 2, ExampleDataset.P.getURI());
        
        Util.equalsSlow(new PathList(expected), new PathList(actual));
    }

    @Test
    public void TaskTwoTestTwo() throws Exception {

    	//k = 3
        //Start or End with P
        //A nach B    	
        List<Path> expected = new LinkedList<>();
        expected.add(new Path(ExampleDataset.A, ExampleDataset.P, ExampleDataset.u3, ExampleDataset.p7, ExampleDataset.B));
        expected.add(new Path(ExampleDataset.A, ExampleDataset.p3, ExampleDataset.u6, ExampleDataset.P, ExampleDataset.B));
        expected.add(new Path(ExampleDataset.A, ExampleDataset.P, ExampleDataset.u3, ExampleDataset.p4, ExampleDataset.u4, ExampleDataset.p5, ExampleDataset.u5, ExampleDataset.p6, ExampleDataset.u3, ExampleDataset.p7, ExampleDataset.B));

    	//omitted:
        //l.add(new Path(ExampleDataset.A, ExampleDataset.p1, ExampleDataset.u1, ExampleDataset.p2, ExampleDataset.u2, ExampleDataset.p8, ExampleDataset.B));
        //because it has to start or end with P
        
        List<Path> actual = pathFinder.find(ExampleDataset.A.getURI(), ExampleDataset.B.getURI(), 3, ExampleDataset.P.getURI());
        
        Util.equalsSlow(new PathList(expected), new PathList(actual));
    }

    @Test
    public void TaskTwoTestOneMoreKThenNecessary() throws Exception {

    	//k = 200
        //Start or End with P
        //A nach B    	
        List<Path> expected = new LinkedList<>();
        expected.add(new Path(ExampleDataset.A, ExampleDataset.P, ExampleDataset.u3, ExampleDataset.p7, ExampleDataset.B));
        expected.add(new Path(ExampleDataset.A, ExampleDataset.p3, ExampleDataset.u6, ExampleDataset.P, ExampleDataset.B));
        expected.add(new Path(ExampleDataset.A, ExampleDataset.P, ExampleDataset.u3, ExampleDataset.p4, ExampleDataset.u4, ExampleDataset.p5, ExampleDataset.u5, ExampleDataset.p6, ExampleDataset.u3, ExampleDataset.p7, ExampleDataset.B));
        
        List<Path> actual = pathFinder.find(ExampleDataset.A.getURI(), ExampleDataset.B.getURI(), 200, ExampleDataset.P.getURI());
        
        Util.equalsSlow(new PathList(expected), new PathList(actual));
    }

}
