package de.dfki.km.academicresearch.eswc2016challenge.unittests.exampleTestTaskTwo;

import de.dfki.km.academicresearch.eswc2016challenge.cameraready.Path;
import de.dfki.km.academicresearch.eswc2016challenge.cameraready.PathFinder;
import de.dfki.km.academicresearch.eswc2016challenge.cameraready.PathList;
import de.dfki.km.academicresearch.eswc2016challenge.cameraready.PrintHeapUsage;
import de.dfki.km.academicresearch.eswc2016challenge.unittests.ConfusionMatrix;
import de.dfki.km.academicresearch.eswc2016challenge.unittests.InputParam;
import de.dfki.km.academicresearch.eswc2016challenge.unittests.PathChooser;
import de.dfki.km.academicresearch.eswc2016challenge.unittests.TaskTest.TaskTwo;
import de.dfki.km.academicresearch.eswc2016challenge.unittests.Util;
import de.dfki.km.academicresearch.eswc2016challenge.unittests.WriteComparableFile;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.Arrays;
import java.util.Collection;
import java.util.List;
import org.apache.jena.riot.RDFDataMgr;
import org.apache.jena.riot.system.StreamRDF;
import org.apache.jena.riot.system.StreamRDFLib;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.junit.runners.Parameterized;
import org.junit.runners.Parameterized.Parameters;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

@RunWith(Parameterized.class)
public class SubsetTask {

    private static final Logger logger = LoggerFactory.getLogger(SubsetTask.class);
    
    //=Choose Implementation=========================================================================
    private static PathFinder pathFinder;
    
    @Before
    public void init() throws Exception {
    	String task = String.format("task2_q%s_%s", query, k);
        logger.debug("@Before test {}.txt", task);
        
        PrintHeapUsage.gcAndprintGB();
        pathFinder = PathChooser.getCurrentPathFinder();
        
        pathFinder.setIndex("challenge_training_dataset", task);
        if(pathFinder.hasIndex()){        	
        	pathFinder.loadIndex(); 
        }
        else{
            StreamRDF streamRDF = StreamRDFLib.sinkTriples(pathFinder);   
            pathFinder.startStreaming();
            RDFDataMgr.parse(streamRDF, String.format("./subset/%s.nt",task)); 
        }
        PrintHeapUsage.gcAndprintGB();
        logger.debug("Finished Importing test {}.txt", task);
    }
    //========================================================================== 
    

    @Parameters(name = "Test {index}:  task2_q{0}_{1}.txt")
    public static Collection<Object[]> data() {
        return Arrays.asList(new Object[][]{
            
            //{1, 32}, //OK 2016-02-27 //after 260 iterations we received 32 valid pathes and H contains 427 entries | improvement: after 135 iterations we received 32 valid pathes and H contains 246 entries
            //{1, 98}, //OK 2016-02-27
            //{1, 1914}, //OK 2016-02-27
            //{1, 16632}, //OK 2016-02-27 but needs some time
            //{1, 212988}, //OK 2016-03-01 (75s - 78s) iteration: 220639 / found pathes: 212988 / extra: 7651 / k: 212988 / nonConstructed: 2319 / used MB: 2043 / P(G).vertices 2781890 / H 450668 / invalid type statistic: {Unkown=0, ShortestPath=5332, Sidetrack=2319, NeverReachDst=0, None=212988, PropertyPathBcOfSidetrack=0, PropertyPathBcOfShortestPath=0}
                         //      iteration: 662000 / found pathes: 63997 / extra: 598003 / k: 212988 / nonConstructed: 220323 / used MB: 3205 / P(G).vertices 10182471 / H 1153551 / invalid type statistic: {NeverReachDst=0, PropertyPathBcOfSidetrack=219596, ShortestPath=1849, None=63997, Sidetrack=727, Unkown=0, PropertyPathBcOfShortestPath=375831}
            
            //{2, 3}, //OK 2016-02-27
            //{2, 4}, //OK 2016-02-27
            //{2, 76}, //OK 2016-02-27
            //{2, 151}, //OK 2016-03-01
            //{2, 2311}, //OK 2016-03-01
            
            //{3, 12}, //OK 2016-02-27 //was first infinite bc of P = "    http://"
            //{3, 76}, //OK 2016-02-27
            //{3, 1440}, //OK 2016-02-27
            //{3, 8088}, //OK 2016-03-01
            
            //{4,      1}, //OK 2016-02-27
            //{4,      6}, //OK 2016-02-27
            //{4,     72}, //OK 2016-02-27
            //{4,    614}, //OK 2016-02-27
            //{4,   5483}, //OK 2016-02-27  //iteration: 6000 / found pathes: 5428 / extra: 572 / k: 5483 / pruned: 238 / used MB: 921 / P(G).vertices 163373 / H 11373 / invalid type statistic: {NeverReachDst=9, PropertyPath=1653, ShortestPath=38577, None=117216, Sidetrack=12678}
            //{4,  52649}, //OK 2016-03-01  //iteration: 50000 / found pathes: 44936 / extra: 5064 / k: 52649 / nonConstructed: 2211 / used MB: 652 / P(G).vertices 1436003 / H 96418 / ....................iteration: 183000 / found pathes: 52307 / extra: 130693 / k: 52649 / nonConstructed: 25772 / used MB: 1828 / P(G).vertices 5164825 / H 337898 / invalid type statistic: {Sidetrack=2624, ShortestPath=3366, None=52307, PropertyPathBcOfShortestPath=101555, Unkown=0, PropertyPathBcOfSidetrack=23148, NeverReachDst=0}
            {4, 471199}    //iteration: 300000 / found pathes: 262729 / extra: 37271 / k: 471199 / nonConstructed: 14572 / used MB: 3272 / P(G).vertices 8889132 / H 578901 / invalid type statistic: {ShortestPath=22699, Sidetrack=14572, Unkown=0, PropertyPathBcOfShortestPath=0, NeverReachDst=0, PropertyPathBcOfSidetrack=0, None=262729}

            
            //==================================================================
            //optimization:
            //color invalid
                //build path
                    //create property
                    //hash set add
            
            //copy clone (we need a fast graph library for adding)
                //add vertex
                //add edge
        });
    }    
    
    

    private int query;
    private int k;    
    
    public SubsetTask(int query, int k) {
        this.query = query;
        this.k = k;
    }

    @Test
    public void test() throws FileNotFoundException, IOException, Exception {

    	String task = "task2_q" + query + "_" + k;
    	
        //load file
        File f = new File("./challenge_training_result_sets/" + task + ".txt");
        Assert.assertTrue("File with expected results should be existing", f.exists());

        //parameters
        InputParam params = TaskTwo.queryToParam.get(query);
        String start = params.getStart();
        String end = params.getEnd();
        String P = params.getP();

        //actual and expected
        List<Path> actual = pathFinder.find(start, end, k, P);
        List<Path> expected = Util.getPaths(f);
        
        PathList listActual = new PathList(actual);
        PathList listExpected = new PathList(expected);
        
        WriteComparableFile.writeSingleFile(new File("./challenge_training_dataset/"), task, listExpected, listActual);
        WriteComparableFile.writeFileToAll(new File("./challenge_training_dataset/"), task, listExpected, listActual);
        
        //Util.equalsSlow(listExpected, listActual);
        
        ConfusionMatrix m = new ConfusionMatrix();
        m.compare(listExpected, listActual);
        m.writeSingleFile(new File("./challenge_training_dataset/"), task);
        //m.printConsole(task);
        m.assertIt();
    }
    
}
