package de.dfki.km.academicresearch.eswc2016challenge.unittests.load;

import de.dfki.km.academicresearch.eswc2016challenge.cameraready.DataBasis;
import de.dfki.km.academicresearch.eswc2016challenge.cameraready.ModifiedEppstein;
import de.dfki.km.academicresearch.eswc2016challenge.cameraready.ModifiedEppsteinBidirectional;
import de.dfki.km.academicresearch.eswc2016challenge.cameraready.PathList;
import java.io.IOException;
import org.junit.BeforeClass;
import org.junit.Test;

/**
 *
 * @author Markus Schr&ouml;der
 */
public class EvalTest {
    
    private static DataBasis data;
    
    public EvalTest() {
    }

    @BeforeClass
    public static void setUpClass() throws IOException, ClassNotFoundException {
        data = new DataBasis("./eval");
        int bp = 0;
    }
    
    /*
        Task1, part1 input parameters
        Start node: "http://dbpedia.org/resource/1952_Winter_Olympics"
        End node: "http://dbpedia.org/resource/Elliot_Richardson"
        k: 377
        ppath expression: !:*
        Dataset: http://aimashup.org/ESWC2016challenge/evaluation_dataset.nt.gz

        Task1, part2 input parameters
        Start node: "http://dbpedia.org/resource/1952_Winter_Olympics"
        End node: "http://dbpedia.org/resource/Elliot_Richardson"
        k: 53008
        ppath expression: !:*
        Dataset: http://aimashup.org/ESWC2016challenge/evaluation_dataset.nt.gz 
     */
    //@Test
    public void task1() throws Exception {
        ModifiedEppstein modepp = new ModifiedEppstein("http://dbpedia.org/resource/Elliot_Richardson", data);
        
        PathList a = modepp.find("http://dbpedia.org/resource/1952_Winter_Olympics", 377);
        a.save("task1-part1-377.txt");
        
        PathList b = modepp.find("http://dbpedia.org/resource/1952_Winter_Olympics", 53008);
        b.save("task1-part2-53008.txt");
    }
    
    /*
        Task2, part1 input parameters
        Start node: "http://dbpedia.org/resource/1952_Winter_Olympics"
        End node: "http://dbpedia.org/resource/Elliot_Richardson"
        k: 374
        ppath expression: (<http://dbpedia.org/property/after>
        Dataset: http://aimashup.org/ESWC2016challenge/evaluation_dataset.nt.gz
    */
    //@Test
    public void task2part1() throws Exception {
        ModifiedEppsteinBidirectional modeppA = new ModifiedEppsteinBidirectional(
                "http://dbpedia.org/resource/1952_Winter_Olympics", 
                "http://dbpedia.org/resource/Elliot_Richardson", 
                "http://dbpedia.org/property/after", 
                374, 
                data);
        
        PathList a = modeppA.find();
        a.save("task2-part1-374.txt");
    }
    
    /*
        Task2, part2 input parameters
        Start node: "http://dbpedia.org/resource/1952_Winter_Olympics"
        End node: "http://dbpedia.org/resource/Elliot_Richardson"
        k: 52664
        ppath expression: (<http://dbpedia.org/property/after>
        Dataset: http://aimashup.org/ESWC2016challenge/evaluation_dataset.nt.gz
    */
    @Test
    public void task2part2() throws Exception {
        ModifiedEppsteinBidirectional modeppA = new ModifiedEppsteinBidirectional(
                "http://dbpedia.org/resource/1952_Winter_Olympics", 
                "http://dbpedia.org/resource/Elliot_Richardson", 
                "http://dbpedia.org/property/after", 
                52664, 
                data);
        
        PathList a = modeppA.find();
        a.save("task2-part2-52664.txt");
    }
    
}
