package de.dfki.km.academicresearch.eswc2016challenge.unittests.load;

import com.google.common.collect.BiMap;
import com.google.common.collect.HashBiMap;
import de.dfki.km.academicresearch.eswc2016challenge.cameraready.DijkstraDistance;
import de.dfki.km.academicresearch.eswc2016challenge.cameraready.PathFinder;
import de.dfki.km.academicresearch.eswc2016challenge.cameraready.PrintHeapUsage;
import de.dfki.km.academicresearch.eswc2016challenge.cameraready.ShortestPathTree;
import de.dfki.km.academicresearch.eswc2016challenge.unittests.PathChooser;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Arrays;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import org.apache.jena.atlas.lib.Sink;
import org.apache.jena.graph.Triple;
import org.apache.jena.riot.RDFDataMgr;
import org.apache.jena.riot.system.StreamRDF;
import org.apache.jena.riot.system.StreamRDFLib;
import org.jgrapht.graph.DirectedWeightedPseudograph;
import org.junit.Test;
import org.nustaq.serialization.FSTObjectInput;
import org.nustaq.serialization.FSTObjectOutput;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 *
 * @author Markus Schr&ouml;der
 */
public class LoadTest {
    
    private static final Logger logger = LoggerFactory.getLogger(LoadTest.class);
    private static PathFinder pathFinder;
    
    private String filename = "evaluation_dataset_filtered";
    //private String filename = "training_dataset_filtered";
    
    public LoadTest() {
    }

    //@Test
    public void parseToNull() throws Exception {
    
        long begin = System.currentTimeMillis();
        System.out.println("begin parse");
        
        RDFDataMgr.parse(StreamRDFLib.sinkNull(), filename + ".nt.gz");
        
        long endparse = System.currentTimeMillis();
        System.out.println((endparse - begin) + " ms parse");
    }
    
    //@Test
    public void parseAndCount() throws Exception {
    
        StreamRDF streamRDF = StreamRDFLib.sinkTriples(new Sink<Triple>() {
            
            Set<String> uris = new HashSet<>();
            int all = 0;
            int spo = 0;
            
            @Override
            public void send(Triple t) {
                if(t.getObject().isURI()) {
                    spo++;
                    uris.add(t.getSubject().getURI());
                    uris.add(t.getPredicate().getURI());
                    uris.add(t.getObject().getURI());
                }
                all++;
            }

            @Override
            public void flush() {
                logger.debug("flush {} SPOs / {} triples | {} uris", spo, all, uris.size());
            }

            @Override
            public void close() {
                
            }
            
        });
        
        long begin = System.currentTimeMillis();
        //System.out.println("begin parse");
        logger.debug("begin parse");
        
        RDFDataMgr.parse(streamRDF, filename + ".nt.gz");
        
        long endparse = System.currentTimeMillis();
        logger.debug("finished in {} ms", (endparse - begin));
        
    }
    
    //@Test
    public void parseAndMapUris() throws Exception {
    
        StreamRDF streamRDF = StreamRDFLib.sinkTriples(new Sink<Triple>() {
            
            Map<String, Integer> uri2id = new HashMap<>();//or BiMap?
            int id = Integer.MIN_VALUE;
            
            @Override
            public void send(Triple t) {
                if(t.getObject().isURI()) {
                    for(String s : new String[] { t.getSubject().getURI(), t.getPredicate().getURI(), t.getObject().getURI() }) {
                        if(!uri2id.containsKey(s)) {
                            uri2id.put(s, id++);
                        }
                    }
                }
            }

            @Override
            public void flush() {
                logger.debug("{} mappings", uri2id.size());
            }

            @Override
            public void close() {
                
            }
            
        });
        
        long begin = System.currentTimeMillis();
        //System.out.println("begin parse");
        logger.debug("begin parse");
        
        RDFDataMgr.parse(streamRDF, filename + ".nt.gz");
        
        long endparse = System.currentTimeMillis();
        logger.debug("finished in {} ms", (endparse - begin));
        
    }
    
    //@Test
    public void parseAndMap() throws Exception {
    
        StreamRDF streamRDF = StreamRDFLib.sinkTriples(new Sink<Triple>() {
            
            BiMap<String, Integer> uri2id = HashBiMap.create(1551041);
            int uriId    = 0;
            int tripleId = 0;
            
            
            //33 724 208 * 4 * (32 bits) =
            //539.587328 megabytes
            
            //33 724 208 * 3 * (32 bits) =
            //404.690496 megabytes
            
            //id ist implizit durch die position im array gegeben
            int[] triplearray = new int[33724208 * 3];
            
            //um duplicate im eval set gleich auszuschließen
            Set<Integer[]> tripleset = new HashSet<>();
            int duplicate = 0;
            
            @Override
            public void send(Triple t) {
                if(t.getObject().isURI()) {
                    
                    //triple
                    int i = 0;
                    Integer[] triple = new Integer[3];
                    
                    //uri to id
                    for(String s : new String[] { t.getSubject().getURI(), t.getPredicate().getURI(), t.getObject().getURI() }) {
                        if(!uri2id.containsKey(s)) {
                            uri2id.put(s, uriId);
                            triplearray[3 * tripleId + i] = triple[i] = uriId;
                            uriId++;
                        } else {
                            triplearray[3 * tripleId + i] = triple[i] = uri2id.get(s);
                        }
                        i++;
                    }
                    
                    //id for the triple
                    if(!tripleset.contains(triple)) {
                        tripleId++;
                    } else {
                        duplicate++;
                    }
                }
            }

            @Override
            public void flush() {
                logger.debug("{} uri mappings, {} duplicates, ids: {}, {}", uri2id.size(), duplicate, uriId, tripleId);
                PrintHeapUsage.printGB();
                
                long begin = System.currentTimeMillis();
                try {
                    FSTObjectOutput fstoo = new FSTObjectOutput(new FileOutputStream(new File("uri2id.bimap")));
                    fstoo.writeObject(uri2id);
                    fstoo.close();
                } catch (IOException ex) {
                    logger.error("", ex);
                }
                long end = System.currentTimeMillis();
                logger.debug("{} ms to save uri2id", end - begin);
                PrintHeapUsage.printGB();
                
                begin = System.currentTimeMillis();
                try {
                    FSTObjectOutput fstoo = new FSTObjectOutput(new FileOutputStream(new File("triple.array")));
                    fstoo.writeObject(triplearray);
                    fstoo.close();
                } catch (IOException ex) {
                    logger.error("", ex);
                }
                end = System.currentTimeMillis();
                logger.debug("{} ms to save triplearray", end - begin);
                PrintHeapUsage.printGB();
            }

            @Override
            public void close() {
                
            }
            
        });
        
        long begin = System.currentTimeMillis();
        //System.out.println("begin parse");
        logger.debug("begin parse");
        PrintHeapUsage.printGB();
        
        RDFDataMgr.parse(streamRDF, filename + ".nt.gz");
        
        long endparse = System.currentTimeMillis();
        logger.debug("finished in {} ms", (endparse - begin));
        
    }
    
    @Test
    public void parseAndMapCorrect() throws Exception {
    
        StreamRDF streamRDF = StreamRDFLib.sinkTriples(new Sink<Triple>() {
            
            BiMap<String, Integer> uri2id = HashBiMap.create(1551041);
            int uriId    = 0;
            int tripleId = 0;
            
            
            //33 724 208 * 4 * (32 bits) =
            //539.587328 megabytes
            
            //33 724 208 * 3 * (32 bits) =
            //404.690496 megabytes
            
            //id ist implizit durch die position im array gegeben
            int[] triplearray = new int[13605253 * 3];//13.605.253
            
            //um duplicate im eval set gleich auszuschließen
            Set<TripleWrapper> tripleset = new HashSet<>();
            int duplicate = 0;
            
            @Override
            public void send(Triple t) {
                if(t.getObject().isURI()) {
                    
                    //triple
                    int i = 0;
                    Integer[] triple = new Integer[3];
                    
                    //uri to id
                    for(String s : new String[] { t.getSubject().getURI(), t.getPredicate().getURI(), t.getObject().getURI() }) {
                        if(!uri2id.containsKey(s)) {
                            uri2id.put(s, uriId);
                            triplearray[3 * tripleId + i] = triple[i] = uriId;
                            uriId++;
                        } else {
                            triplearray[3 * tripleId + i] = triple[i] = uri2id.get(s);
                        }
                        i++;
                    }
                    
                    TripleWrapper tw = new TripleWrapper(triple);
                    
                    //id for the triple
                    if(!tripleset.contains(tw)) {
                        tripleId++;
                        tripleset.add(tw);
                    } else {
                        duplicate++;
                    }
                }
            }

            @Override
            public void flush() {
                logger.debug("{} uri mappings, {} duplicates, ids: {}, {}", uri2id.size(), duplicate, uriId, tripleId);
                PrintHeapUsage.printGB();
                
                long begin = System.currentTimeMillis();
                try {
                    FSTObjectOutput fstoo = new FSTObjectOutput(new FileOutputStream(new File("uri2id.bimap")));
                    fstoo.writeObject(uri2id);
                    fstoo.close();
                } catch (IOException ex) {
                    logger.error("", ex);
                }
                long end = System.currentTimeMillis();
                logger.debug("{} ms to save uri2id", end - begin);
                PrintHeapUsage.printGB();
                
                begin = System.currentTimeMillis();
                try {
                    FSTObjectOutput fstoo = new FSTObjectOutput(new FileOutputStream(new File("triple.array")));
                    fstoo.writeObject(triplearray);
                    fstoo.close();
                } catch (IOException ex) {
                    logger.error("", ex);
                }
                end = System.currentTimeMillis();
                logger.debug("{} ms to save triplearray", end - begin);
                PrintHeapUsage.printGB();
            }

            @Override
            public void close() {
                
            }
            
        });
        
        long begin = System.currentTimeMillis();
        //System.out.println("begin parse");
        logger.debug("begin parse");
        PrintHeapUsage.printGB();
        
        RDFDataMgr.parse(streamRDF, filename + ".nt.gz");
        
        long endparse = System.currentTimeMillis();
        logger.debug("finished in {} ms", (endparse - begin));
        
    }
    
    class TripleWrapper {
        
        Integer[] triple;

        public TripleWrapper(Integer[] triple) {
            this.triple = triple;
        }

        @Override
        public int hashCode() {
            return Arrays.hashCode(triple);
        }

        @Override
        public boolean equals(Object obj) {
            if (this == obj) {
                return true;
            }
            if (obj == null) {
                return false;
            }
            if (getClass() != obj.getClass()) {
                return false;
            }
            final TripleWrapper other = (TripleWrapper) obj;
            if (!Arrays.equals(this.triple, other.triple)) {
                return false;
            }
            return true;
        }
        
    }
    
    //@Test
    public void load() throws Exception {
        
        long begin = System.currentTimeMillis();
        FSTObjectInput fstui = new FSTObjectInput(new FileInputStream(new File("uri2id.bimap")));
        BiMap<String, Integer> uri2id = (BiMap<String, Integer>) fstui.readObject();
        fstui.close();
        long end = System.currentTimeMillis();
        
        logger.debug("{} uri mappings in {} ms loaded", uri2id.size(), end - begin);
        
        begin = System.currentTimeMillis();
        FSTObjectInput fstta = new FSTObjectInput(new FileInputStream(new File("triple.array")));
        int[] triplearray = (int[]) fstta.readObject();
        fstta.close();
        end = System.currentTimeMillis();
        
        logger.debug("{} sized array in {} ms loaded", triplearray.length, end - begin);
        
        System.gc();
        PrintHeapUsage.printGB();
        
        //JGraphT
        begin = System.currentTimeMillis();
        
        DirectedWeightedPseudograph<Integer, Integer> g = new DirectedWeightedPseudograph<>(Integer.class);
        
        //0 bis uri2id.size()
        //for(Integer v : uri2id.values()) {
        //    g.addVertex(v);
        //}
        //da wir beim generieren von 0 hochzählen
        for(int i = 0; i < uri2id.size(); i++) {
            g.addVertex(i);
        }
        
        for(int i = 0; i < triplearray.length; i += 3) {
            //position dient als eindeutigen identifier für die edge
            //The source and target vertices must already be contained in this graph.
            g.addEdge(triplearray[i+0], triplearray[i+2], i);
            //p wird ignoriert da spo zusammen relevant ist was durch i eindeutig identifizierbar ist
            //zudem kann man über die position auf triplearray zugreifen
        }
        
        end = System.currentTimeMillis();
        
        logger.debug("jgrapht's DirectedWeightedPseudograph with {} vertices and {} edges in {} ms", g.vertexSet().size(), g.edgeSet().size(), end - begin);
        //System.gc();
        PrintHeapUsage.printGB();
        
        logger.debug("0's outgoing edges are {}", g.outgoingEdgesOf(0));
        logger.debug("820564's outgoing edges are {}", g.outgoingEdgesOf(820564));
    }
    
    //@Test
    public void loadAll() throws Exception {
        
        FSTObjectInput fstui = new FSTObjectInput(new FileInputStream(new File("uri2id.bimap")));
        BiMap<String, Integer> uri2id = (BiMap<String, Integer>) fstui.readObject();
        fstui.close();
        
        FSTObjectInput fstta = new FSTObjectInput(new FileInputStream(new File("triple.array")));
        int[] triplearray = (int[]) fstta.readObject();
        fstta.close();
        
        DirectedWeightedPseudograph<Integer, Integer> g = new DirectedWeightedPseudograph<>(Integer.class);
        for(int i = 0; i < uri2id.size(); i++) {
            g.addVertex(i);
        }
        for(int i = 0; i < triplearray.length; i += 3) {
            g.addEdge(triplearray[i+0], triplearray[i+2], i);
        }
        
        int redbullId = uri2id.get("http://dbpedia.org/resource/Red_Bull");
        ShortestPathTree<Integer, Integer> T = new DijkstraDistance<Integer, Integer>().computeShortestPathTree(g, redbullId);
        
        logger.debug("{}", T);
    }
    
    //@Test
    public void parseToPathFinder() throws Exception {
    
        pathFinder = PathChooser.getCurrentPathFinder();

        pathFinder.setIndex("challenge_evaluation_dataset", "evaluation_dataset");
        
        StreamRDF streamRDF = StreamRDFLib.sinkTriples(pathFinder);
        pathFinder.startStreaming();
        
        long begin = System.currentTimeMillis();
        System.out.println("begin parse");
        
        RDFDataMgr.parse(streamRDF, "evaluation_dataset_filtered.nt.gz");
        
        long endparse = System.currentTimeMillis();
        System.out.println((endparse - begin) + " ms parse");
    }
}
