package de.dfki.km.academicresearch.eswc2016challenge.unittests.load;


import de.dfki.km.academicresearch.eswc2016challenge.cameraready.DataBasis;
import de.dfki.km.academicresearch.eswc2016challenge.cameraready.ModifiedEppstein;
import de.dfki.km.academicresearch.eswc2016challenge.cameraready.PathList;
import de.dfki.km.academicresearch.eswc2016challenge.unittests.ConfusionMatrix;
import de.dfki.km.academicresearch.eswc2016challenge.unittests.InputParam;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.Arrays;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;
import org.junit.Assert;
import org.junit.BeforeClass;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.junit.runners.Parameterized;
import org.junit.runners.Parameterized.Parameters;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

@RunWith(Parameterized.class)
public class TaskOneModEpp {

    private static final Logger logger = LoggerFactory.getLogger(TaskOneModEpp.class);
    
    private static DataBasis data;
    
    @BeforeClass
    public static void onceExecutedBeforeAll() throws Exception {
        data = new DataBasis("./train");
    }
    //========================================================================== 

    @Parameters(name = "Test {index}:  task1_q{0}_{1}.txt")
    public static Collection<Object[]> data() {
        return Arrays.asList(new Object[][]{
            
            //{1, 8}, //OK 2016-04-19             
            //{1, 344}, //OK 2016-04-19            
            //{1, 1068}, //OK 2016-04-19
            //{1, 20152}, //OK 2016-04-19
            
            
            //{2, 3},
            //{2, 4},
            //{2, 79},
            //{2, 154},
            
            //{3, 36},
            //{3, 336},
            //{3, 4866},
            
            
            //{4, 2},
            //{4, 16},
            //{4, 250},
            //{4, 1906},
            //{4, 20224}, 
            {4, 175560} //OK 2016-04-19
            
        });
    }

    private int query;
    private int k;    
    public TaskOneModEpp(int query, int k) {
        this.query = query;
        this.k = k;
    }

    @Test
    public void test() throws FileNotFoundException, IOException, Exception {

        //expected outcome
        File f = new File("./challenge_training_result_sets/task1_q" + query + "_" + k + ".txt");
        Assert.assertTrue("File with expected results should be existing", f.exists());
        PathList expected = new PathList(f);
        
        //parameters
        InputParam params = queryToParam.get(query);
        String start = params.getStart();
        String end = params.getEnd();

        
        String task = "task1_q" + query + "_" + k;
        logger.debug("start {}", task);
        
        //our algo
        ModifiedEppstein modepp = new ModifiedEppstein(end, data);
        PathList actual = modepp.find(start, k);
        
        //compare
        ConfusionMatrix m = new ConfusionMatrix();
        m.compare(expected, actual);
        m.writeSingleFile(new File("./challenge_evaluation_dataset/"), task);
        m.assertIt();
        
        logger.debug("end {}", task);
    }
    
    //PARAM
    public static Map<Integer, InputParam> queryToParam = generateParam();
    private static Map<Integer, InputParam> generateParam() {
        Map<Integer, InputParam> map = new HashMap<>();
        map.put(1, new InputParam("http://dbpedia.org/resource/Felipe_Massa", "http://dbpedia.org/resource/Red_Bull", null));
        map.put(2, new InputParam("http://dbpedia.org/resource/1952_Winter_Olympics", "http://dbpedia.org/resource/Elliot_Richardson", null));
        map.put(3, new InputParam("http://dbpedia.org/resource/Karl_W._Hofmann", "http://dbpedia.org/resource/Elliot_Richardson", null));
        map.put(4, new InputParam("http://dbpedia.org/resource/James_K._Polk", "http://dbpedia.org/resource/Felix_Grundy", null));
        return map;
    }
}
